@@ -16,9 +16,13 @@
 import json
 import logging
 from flask import Flask, request
+import ferry.install
 from ferry.install import Installer
 from ferry.docker.manager import DockerManager
 from ferry.docker.docker import DockerInstance
+import Queue
+import threading2
+import time
 
 # Initialize Flask
 app = Flask(__name__)
@@ -27,71 +31,57 @@ app = Flask(__name__)
 installer = Installer()
 docker = DockerManager()
 
-"""
-Fetch the current information for a particular filesystem. 
-"""
-@app.route('/storage', methods=['GET'])
-def query_storage():
-    status = AllocationResponse()
-    status.uuid = request.args['uuid']
-    status.status = status.NOT_EXISTS
+def _stack_worker():
+    """
+    Worker thread.
+    """
+    while(True):
+        payload = _new_queue.get()
 
-    # Get the time the storage cluster was created, 
-    # along with basic usage information. 
-    info = storage.query_storage(status.uuid)
-    if info != None:
-        status.status = info
+        if payload["_action"] == "new":
+            _allocate_new_worker(payload["_uuid"], payload)
+        elif payload["_action"] == "stopped":
+            _allocate_stopped_worker(payload)
+        elif payload["_action"] == "snapshotted":
+            _allocate_snapshot_worker(payload["_uuid"], payload)
+        elif payload["_action"] == "manage":
+            _manage_stack_worker(payload["_uuid"], payload["_manage"], payload["_key"])
+            
+        time.sleep(2)
 
-    # Return the JSON reply.
-    return status.json()
+_new_queue = Queue.Queue()
+_new_stack_worker = threading2.Thread(target=_stack_worker)
+_new_stack_worker.daemon = True
+_new_stack_worker.start()
 
-
-"""
-Fetch the current docker version
-"""
-@app.route('/version', methods=['GET'])
-def get_version():
-    return docker.version()
-
-def _get_private_key(keydir, public_key):
-    return keydir + "/" + public_key + ".pem"
-
-"""
-Allocate the backend from a snapshot. 
-"""
-def _allocate_backend_from_snapshot(payload, key_name):
+def _allocate_backend_from_snapshot(cluster_uuid, payload, key_name):
+    """
+    Allocate the backend from a snapshot. 
+    """
     snapshot_uuid = payload['_file']
     backends = docker.fetch_snapshot_backend(snapshot_uuid)
 
     if backends:
-        return _allocate_backend(payload = None,
+        return _allocate_backend(cluster_uuid = cluster_uuid,
+                                 payload = None,
                                  key_name = key_name, 
-                                 backends = backends)
+                                 backends = backends,
+                                 new_stack = True)
 
-"""
-Allocate the backend from a deployment. 
-"""
-def _allocate_backend_from_deploy(payload, key_name, params=None):
-    app_uuid = payload['_file']
-    backends = docker.fetch_deployed_backend(app_uuid, params)
-    
-    if backends:
-        return _allocate_backend(payload = None,
-                                 key_name = key_name, 
-                                 backends = backends)
-
-"""
-Allocate the backend from a stopped service. 
-"""
 def _allocate_backend_from_stopped(payload):
+    """
+    Allocate the backend from a stopped service. 
+    """
     app_uuid = payload['_file']
     backends, key_name = docker.fetch_stopped_backend(app_uuid)
     
     if backends:
-        backend_info, backend_plan = _allocate_backend(payload = None,
+        backend_info, backend_plan = _allocate_backend(cluster_uuid = app_uuid,
+                                                       payload = None,
                                                        key_name = key_name, 
                                                        backends = backends,
-                                                       uuid = app_uuid)
+                                                       uuid = app_uuid,
+                                                       new_stack = False)
         return backend_info, backend_plan, key_name
 
 def _fetch_num_instances(instance_arg):
@@ -104,7 +94,7 @@ def _fetch_num_instances(instance_arg):
 
     return reply
 
-def _allocate_compute(computes, key_name, storage_uuid):
+def _allocate_compute(cluster_uuid, computes, key_name, storage_uuid):
     """
     Allocate a new compute backend. This method assumes that every
     compute backend already has a specific instance count associated
@@ -126,20 +116,27 @@ def _allocate_compute(computes, key_name, storage_uuid):
         if 'layers' in c:
             layers = c['layers']
 
-        compute_uuid, compute_containers = docker.allocate_compute(compute_type = compute_type,
+        compute_uuid, compute_containers = docker.allocate_compute(cluster_uuid = cluster_uuid,
+                                                                   compute_type = compute_type,
                                                                    key_name = key_name,
                                                                    storage_uuid = storage_uuid, 
                                                                    args = args, 
                                                                    num_instances = num_instances,
                                                                    layers = layers)
-        compute_plan.append( { 'uuid' : compute_uuid,
-                               'containers' : compute_containers,
-                               'type' : compute_type, 
-                               'start' : 'start' } )
-        uuids.append( compute_uuid )
+        if compute_uuid:
+            compute_plan.append( { 'uuid' : compute_uuid,
+                                   'containers' : compute_containers,
+                                   'type' : compute_type, 
+                                   'start' : 'start' } )
+            uuids.append( compute_uuid )
+        else:
+            # The manager could not allocate the compute backend
+            # properly. Return a failure so that we can update the
+            # status properly and cancel the stack.
+            return None, None
     return uuids, compute_plan
 
-def _restart_compute(computes):
+def _restart_compute(cluster_uuid, computes):
     uuids = []
     compute_plan = []
     for c in computes:
@@ -155,17 +152,19 @@ def _restart_compute(computes):
                                'containers' : containers,
                                'type' : compute_type, 
                                'start' : 'restart' } )
-        docker.restart_containers(service_uuid, containers)
+        docker.restart_containers(cluster_uuid, service_uuid, containers)
     return uuids, compute_plan
 
-"""
-Allocate a brand new backend
-"""
-def _allocate_backend(payload,
+def _allocate_backend(cluster_uuid,
+                      payload,
                       key_name, 
                       backends=None,
                       replace=False,
-                      uuid=None):
+                      uuid=None,
+                      new_stack = True):
+    """
+    Allocate a brand new backend
+    """
     if not backends:
         # We should find the backend information in the payload. 
         if 'backend' in payload:
@@ -189,7 +188,7 @@ def _allocate_backend(payload,
     # we create a fresh backend. 
     for b in backends:
         storage = b['storage']
-        if not uuid:
+        if new_stack:
             args = None
             if 'args' in storage:
                 args = storage['args']
@@ -201,16 +200,24 @@ def _allocate_backend(payload,
             if 'layers' in storage:
                 layers = storage['layers']
 
-            storage_uuid, storage_containers = docker.allocate_storage(storage_type = storage_type, 
+            storage_uuid, storage_containers = docker.allocate_storage(cluster_uuid = cluster_uuid,
+                                                                       storage_type = storage_type, 
                                                                        key_name = key_name, 
                                                                        num_instances = num_instances,
                                                                        layers = layers,
                                                                        args = args,
                                                                        replace = replace)
-            storage_plan.append( { 'uuid' : storage_uuid,
-                                   'containers' : storage_containers,
-                                   'type' : storage_type, 
-                                   'start' : 'start' } )
+
+            if storage_uuid:
+                storage_plan.append( { 'uuid' : storage_uuid,
+                                       'containers' : storage_containers,
+                                       'type' : storage_type, 
+                                       'start' : 'start' } )
+            else:
+                # The storage was not allocated properly. Change 
+                # the status so that the stack can be properly cancelled. 
+                backend_info["status"] = 'failed'
+                return backend_info, None
         else:
             storage_uuid = storage['uuid']
             storage_type = storage['type']
@@ -222,20 +229,28 @@ def _allocate_backend(payload,
                                    'containers' : containers,
                                    'type' : storage_type, 
                                    'start' : 'restart' } )
-            docker.restart_containers(storage_uuid, containers)
+            docker.restart_containers(cluster_uuid, storage_uuid, containers)
                                                   
         # Now allocate the compute backend. The compute is optional so
         # we should check if it even exists first. 
         compute_uuid = []
         if 'compute' in b:
             if not uuid:
-                compute_uuid, plan = _allocate_compute(computes = b['compute'], 
+                compute_uuid, plan = _allocate_compute(cluster_uuid = cluster_uuid,
+                                                       computes = b['compute'], 
                                                        key_name = key_name, 
                                                        storage_uuid = storage_uuid) 
-                compute_uuids += compute_uuid
-                compute_plan += plan
+
+                if compute_uuid:
+                    compute_uuids += compute_uuid
+                    compute_plan += plan
+                else:
+                    # The storage was not allocated properly. Change 
+                    # the status so that the stack can be properly cancelled. 
+                    backend_info["status"] = 'failed'
+                    return backend_info, None
             else:
-                compute_uuid, plan = _restart_compute(b['compute'])
+                compute_uuid, plan = _restart_compute(cluster_uuid, b['compute'])
                 compute_uuids += compute_uuid
                 compute_plan += plan
 
@@ -244,7 +259,7 @@ def _allocate_backend(payload,
     return backend_info, { 'storage' : storage_plan,
                            'compute' : compute_plan }
 
-def _allocate_connectors(payload, key_name, backend_info):
+def _allocate_connectors(cluster_uuid, payload, key_name, backend_info):
     connector_info = []
     connector_plan = []
     if 'connectors' in payload:
@@ -258,7 +273,6 @@ def _allocate_connectors(payload, key_name, backend_info):
             # Check if this connector type has already been pulled
             # into the local index. If not, manually pull it. 
             connector_type = c['personality']
-            logging.warning("allocating %d instances of %s" % (num_instances, connector_type))
             if not installer._check_and_pull_image(connector_type):
                 # We could not fetch this connetor. Instead of 
                 # finishing, just return an error.
@@ -281,52 +295,50 @@ def _allocate_connectors(payload, key_name, backend_info):
                 else:
                     args = {}
 
-                # The user can choose to expose ports on the connectors.                
+                # The user can choose to expose ports on the connectors.
                 if 'ports' in c:
                     ports = c['ports']
                 else:
                     ports = []
 
                 # Now allocate the connector. 
-                uuid, containers = docker.allocate_connector(connector_type = connector_type,
+                uuid, containers = docker.allocate_connector(cluster_uuid = cluster_uuid,
+                                                             connector_type = connector_type,
                                                              key_name = key_name, 
                                                              backend = backend_info, 
                                                              name = connector_name, 
                                                              args = args,
                                                              ports = ports)
-                connector_plan.append( { 'uuid' : uuid,
-                                         'containers' : containers,
-                                         'type' : connector_type, 
-                                         'start' : 'start' } )
-                connector_info.append(uuid)
+                if uuid:
+                    connector_plan.append( { 'uuid' : uuid,
+                                             'containers' : containers,
+                                             'type' : connector_type, 
+                                             'start' : 'start' } )
+                    connector_info.append(uuid)
+                else:
+                    # The manager could not allocate the connectors
+                    # properly. Return a failure so that we can update the
+                    # status properly and cancel the stack.
+                    return Fail, [], connector_plan
+                    
     return True, connector_info, connector_plan
 
-"""
-Allocate the connectors from a snapshot. 
-"""
-def _allocate_connectors_from_snapshot(payload, key_name, backend_info):
+def _allocate_connectors_from_snapshot(cluster_uuid, payload, key_name, backend_info):
+    """
+    Allocate the connectors from a snapshot. 
+    """
     snapshot_uuid = payload['_file']
-    return docker.allocate_snapshot_connectors(snapshot_uuid,
+    return docker.allocate_snapshot_connectors(cluster_uuid,
+                                               snapshot_uuid,
                                                key_name, 
                                                backend_info)
 
-"""
-Allocate the connectors from a deployment. 
-"""
-def _allocate_connectors_from_deploy(payload, key_name, backend_info, params=None):
-    app_uuid = payload['_file']
-    return docker.allocate_deployed_connectors(app_uuid,
-                                               key_name, 
-                                               backend_info,
-                                               params)
-
-"""
-Allocate the connectors from a stopped application. 
-"""
-def _allocate_connectors_from_stopped(payload, key_name, backend_info, params=None):
+def _allocate_connectors_from_stopped(payload, backend_info, params=None):
+    """
+    Allocate the connectors from a stopped application. 
+    """
     app_uuid = payload['_file']
     return docker.allocate_stopped_connectors(app_uuid,
-                                              key_name, 
                                               backend_info,
                                               params)
 
@@ -336,21 +348,22 @@ def _register_ip_addresses(backend_plan, connector_plan):
     of all the containers. 
     """
     ips = []
+    private_key = None
     for s in backend_plan['storage']:
         for c in s['containers']:
             if isinstance(c, dict):
-                ips.append( [c['internal_ip'], c['hostname']] )
+                ips.append( [c['internal_ip'], c['external_ip'], c['hostname']] )
                 private_key = c['privatekey']
             else:
-                ips.append( [c.internal_ip, c.host_name] )
+                ips.append( [c.internal_ip, c.external_ip, c.host_name] )
                 private_key = c.privatekey
 
     for s in backend_plan['compute']:
         for c in s['containers']:
             if isinstance(c, dict):
-                ips.append( [c['internal_ip'], c['hostname']] )
+                ips.append( [c['internal_ip'], c['external_ip'], c['hostname']] )
             else:
-                ips.append( [c.internal_ip, c.host_name] )
+                ips.append( [c.internal_ip, c.external_ip, c.host_name] )
     for s in connector_plan:
         for c in s['containers']:
             # This is slightly awkward. It is because when starting
@@ -358,10 +371,14 @@ def _register_ip_addresses(backend_plan, connector_plan):
             # when restarting we get dictionary descriptions. Should just
             # fix at the restart level! 
             if isinstance(c, dict):
-                ips.append( [c['internal_ip'], c['hostname']] )
+                ips.append( [c['internal_ip'], c['external_ip'], c['hostname']] )
             else:
-                ips.append( [c.internal_ip, c.host_name] )
-    docker._transfer_ip(private_key, ips)
+                ips.append( [c.internal_ip, c.external_ip, c.host_name] )
+
+    # It's possible that the storage wasn't allocated
+    # properly and so there's nothing to transfer. 
+    if private_key:
+        docker._transfer_ip(private_key, ips)
 
 def _start_all_services(backend_plan, connector_plan):
     """
@@ -413,47 +430,114 @@ def _allocate_new(payload, key_name):
 
     # Now allocate the backend. This includes both storage and compute. 
     reply = {}
-    backend_info, backend_plan = _allocate_backend(payload = payload, 
-                                                   key_name = key_name,
-                                                   replace=True)
+    uuid = docker.reserve_stack()
+    payload["_action"] = "new"
+    payload["_uuid"] = str(uuid)
+    payload["_key"] = key_name
+    _new_queue.put(payload)
+    docker.register_stack(backends = { 'uuids':[] }, 
+                          connectors = [], 
+                          base = payload['_file'], 
+                          cluster_uuid = uuid, 
+                          status='building',
+                          key = key_name,
+                          new_stack=True)
 
+    return json.dumps({ 'text' : str(uuid),
+                        'status' : 'building' })
+
+def _cancel_stack(uuid, backend_info, connector_info, base):
+    logging.info("canceling stack...")
+    docker.cancel_stack(uuid, backend_info, connector_info)
+    docker.register_stack(backends = { 'uuids':[] }, 
+                          connectors = [], 
+                          base = base, 
+                          cluster_uuid = uuid, 
+                          status='failed', 
+                          new_stack=False)
+
+def _allocate_new_worker(uuid, payload):
+    """
+    Helper function to allocate and start a new stack. 
+    """
+    reply = {}
+    key_name = payload['_key']
+    logging.info("creating backend...")
+    backend_info, backend_plan = _allocate_backend(cluster_uuid = uuid,
+                                                   payload = payload, 
+                                                   key_name = key_name,
+                                                   replace=True,
+                                                   new_stack=True)
     # Check if the backend status was ok, and if so,
     # go ahead and allocate the connectors. 
     reply['status'] = backend_info['status']
     if backend_info['status'] == 'ok':
-        success, connector_info, connector_plan = _allocate_connectors(payload = payload, 
+        logging.info("creating connectors...")
+        success, connector_info, connector_plan = _allocate_connectors(cluster_uuid = uuid,
+                                                                       payload = payload, 
                                                                        key_name = key_name, 
                                                                        backend_info = backend_info['uuids'])
 
         if success:
+            logging.info("starting services...")
             output = _start_all_services(backend_plan, connector_plan)
-            uuid = docker.register_stack(backends = backend_info, 
-                                         connectors = connector_info, 
-                                         base = payload['_file'], 
-                                         key = key_name)
+            docker.register_stack(backends = backend_info, 
+                                  connectors = connector_info, 
+                                  base = payload['_file'], 
+                                  key = key_name,
+                                  cluster_uuid = uuid, 
+                                  status='running', 
+                                  new_stack=False)
             reply['text'] = str(uuid)
             reply['msgs'] = output
         else:
             # One or more connectors was not instantiated properly. 
-            docker.cancel_stack(backend_info, connector_info)
+            _cancel_stack(uuid, backend_info, connector_info, payload['_file'])
             reply['status'] = 'failed'
+    else:
+        _cancel_stack(uuid, backend_info, [], payload['_file'])
+        reply['status'] = 'failed'
 
     return json.dumps(reply)
 
 def _allocate_stopped(payload):
+    uuid = payload['_file']
+    stack = docker.get_stack(uuid)
+    payload["_action"] = "stopped"
+    payload["_key"] = stack['key']
+    _new_queue.put(payload)
+    docker.register_stack(backends = stack['backends'], 
+                          connectors = stack['connectors'],
+                          base = stack['base'],
+                          cluster_uuid = uuid,
+                          status='restarting', 
+                          key = stack['key'],
+                          new_stack = False)
+    return json.dumps({'status' : 'building',
+                       'text' : str(uuid)})
+
+def _allocate_stopped_worker(payload):
     """
     Helper function to allocate and start a stopped stack. 
     """
-    base = docker.get_base_image(payload['_file'])
-    backend_info, backend_plan, key_name = _allocate_backend_from_stopped(payload)
+    uuid = payload['_file']
+    stack = docker.get_stack(uuid)
+    logging.info("creating backend...")
+    backend_info, backend_plan, key_name = _allocate_backend_from_stopped(payload = payload)
+                                                                          
     if backend_info['status'] == 'ok':
-        connector_info, connector_plan = _allocate_connectors_from_stopped(payload, key_name, backend_info['uuids'])
-        output = _start_all_services(backend_plan, connector_plan)        
-        uuid = docker.register_stack(backends = backend_info,
-                                     connectors = connector_info,
-                                     base = base,
-                                     uuid = payload['_file'],
-                                     key = key_name)
+        logging.info("creating connectors...")
+        connector_info, connector_plan = _allocate_connectors_from_stopped(payload = payload, 
+                                                                           backend_info = backend_info['uuids'])
+        logging.info("starting services...")
+        output = _start_all_services(backend_plan, connector_plan)
+        docker.register_stack(backends = backend_info,
+                              connectors = connector_info,
+                              base = stack['base'],
+                              cluster_uuid = uuid,
+                              status='running', 
+                              key = stack['key'],
+                              new_stack = False)
         return json.dumps({'status' : 'ok',
                            'text' : str(uuid),
                            'msgs' : output})
@@ -464,41 +548,73 @@ def _allocate_snapshot(payload, key_name):
     """
     Helper function to allocate and start a snapshot.
     """
-    backend_info, backend_plan = _allocate_backend_from_snapshot(payload, key_name)
+    uuid = docker.reserve_stack()
+    payload["_action"] = "snapshotted"
+    payload["_uuid"] = str(uuid)
+    payload["_key"] = key_name
+    _new_queue.put(payload)
+    docker.register_stack(backends = { 'uuids':[] }, 
+                          connectors = [], 
+                          base = payload['_file'], 
+                          cluster_uuid = uuid, 
+                          status='building',
+                          key = key_name,
+                          new_stack=True)
+    return json.dumps({ 'text' : str(uuid),
+                        'status' : 'building' })
+
+def _allocate_snapshot_worker(uuid, payload):
+    """
+    Helper function to allocate and start a snapshot.
+    """
+    key_name = payload['_key']
+    backend_info, backend_plan = _allocate_backend_from_snapshot(cluster_uuid = uuid,
+                                                                 payload = payload,
+                                                                 key_name = key_name)
     if backend_info['status'] == 'ok':
-        connector_info, connector_plan = _allocate_connectors_from_snapshot(payload, 
-                                                                            key_name,
-                                                                            backend_info['uuids'])
+        connector_info, connector_plan = _allocate_connectors_from_snapshot(cluster_uuid = uuid,
+                                                                            payload = payload, 
+                                                                            key_name = key_name,                                                                            
+                                                                            backend_info = backend_info['uuids'])
         output = _start_all_services(backend_plan, connector_plan)
-        uuid = docker.register_stack(backends = backend_info, 
-                                     connectors = connector_info, 
-                                     base = payload['_file'],
-                                     key = key_name)
+        docker.register_stack(backends = backend_info, 
+                              connectors = connector_info, 
+                              base = payload['_file'],
+                              cluster_uuid = uuid,
+                              status='running', 
+                              key = key_name,
+                              new_stack = True)
         return json.dumps({'status' : 'ok',
                            'text' : str(uuid),
                            'msgs' : output })
     else:
         return json.dumps({'status' : 'failed'})
 
-def _allocate_deployed(payload, key_name, params=None):
-    """
-    Helper function to allocate and start a deployed application. 
-    """
-    backend_info, backend_plan = _allocate_backend_from_deploy(payload, key_name, params)
-    if backend_info['status'] == 'ok':
-        connector_info, connector_plan = _allocate_connectors_from_deploy(payload, 
-                                                                          key_name, 
-                                                                          backend_info['uuids'],
-                                                                          params)
-        output = _start_all_services(backend_plan, connector_plan)
-        uuid = docker.register_stack(backends = backend_info, 
-                                     connectors = connector_info, 
-                                     base = payload['_file'],
-                                     key = key_name)
-        return json.dumps({'status' : 'ok',
-                           'text' : str(uuid),
-                           'msgs' : output })
 
+@app.route('/storage', methods=['GET'])
+def query_storage():
+    """
+    Fetch the current information for a particular filesystem. 
+    """
+    status = AllocationResponse()
+    status.uuid = request.args['uuid']
+    status.status = status.NOT_EXISTS
+
+    # Get the time the storage cluster was created, 
+    # along with basic usage information. 
+    info = storage.query_storage(status.uuid)
+    if info != None:
+        status.status = info
+
+    # Return the JSON reply.
+    return status.json()
+
+@app.route('/version', methods=['GET'])
+def get_version():
+    """
+    Fetch the current docker version
+    """
+    return docker.version()
 
 @app.route('/login', methods=['POST'])
 def login_registry():
@@ -545,7 +661,6 @@ def allocate_stack():
     mode = request.form['mode']
     conf = request.form['conf']
     key_name = request.form['key']
-    params = docker._get_deploy_params(mode, conf)
 
     # Check whether the user wants to start from fresh or
     # start with a snapshot.
@@ -553,23 +668,18 @@ def allocate_stack():
         return _allocate_stopped(payload)
     elif docker.is_snapshot(payload['_file']):
         return _allocate_snapshot(payload, key_name)
-    elif docker.is_deployed(payload['_file'], params):
-        return _allocate_deployed(payload, params)
     elif '_file_path' in payload:
         return _allocate_new(payload, key_name)
     else:
         return "Could not start " + payload['_file']
 
-@app.route('/deployed', methods=['GET'])
-def query_deployed():
+@app.route('/quit', methods=['POST'])
+def quit():
     """
-    Query the deployed applications.
+    Quit any backend services that may be running. 
     """
-    mode = request.args['mode']
-    conf = request.args['conf']
-    params = docker._get_deploy_params(mode, conf)
-
-    return docker.query_deployed(params)
+    docker.quit()
+    return ""
 
 @app.route('/query', methods=['GET'])
 def query_stacks():
@@ -602,48 +712,49 @@ def inspect():
     Inspect a particular stack.
     """
     uuid = request.args['uuid']
+
     if docker.is_running(uuid) or docker.is_stopped(uuid):
+        # This is an active application stack. 
         return docker.inspect_stack(uuid)
     elif docker.is_installed(uuid):
+        # This is the name of an installed application.
         return docker.inspect_installed(uuid)
+    else:
+        # Try to see if this is an active service.
+        service = docker._get_service_configuration(uuid, detailed=True)
+        if service:
+            service_info = docker._get_inspect_info(uuid)
+            return json.dumps(service_info,
+                              sort_keys=True,
+                              indent=2,
+                              separators=(',',':'))
+    return "could not inspect " + str(uuid)
 
-"""
-Copy over logs
-"""
 @app.route('/logs', methods=['GET'])
 def logs():
+    """
+    Copy over logs
+    """
     stack_uuid = request.args['uuid']
     to_dir = request.args['dir']
     return docker.copy_logs(stack_uuid, to_dir)
 
-"""
-"""
-@app.route('/deploy', methods=['POST'])
-def deploy_stack():
-    stack_uuid = request.form['uuid']
-    mode = request.form['mode']
-    conf = request.form['conf']
-
-    # Deploy the stack and check if we need to
-    # automatically start the stack as well. 
-    params = docker._get_deploy_params(mode, conf)
-    if docker.deploy_stack(stack_uuid, params):
-        _allocate_deployed( { '_file' : stack_uuid } )
-
-"""
-Manage the stacks.
-"""
 @app.route('/manage/stack', methods=['POST'])
 def manage_stack():
-    stack_uuid = request.form['uuid']
-    stack_action = request.form['action']
-    private_key = request.form['key']
-    reply = docker.manage_stack(stack_uuid = stack_uuid, 
-                                private_key = private_key,
-                                action = stack_action)
+    """
+    Manage the stacks.
+    """
+    payload = { "_uuid" : request.form['uuid'],
+                "_manage" : request.form['action'],
+                "_key" : request.form['key'],
+                "_action" : "manage" }
+    _new_queue.put(payload)
+    return ""
 
-    # Format the message to make more sense.
-    if reply['status']:
-        return reply['msg'] + ' ' + reply['uuid']
-    else:
-        return reply['msg']
+def _manage_stack_worker(uuid, action, private_key):
+    """
+    Manage the stacks.
+    """
+    reply = docker.manage_stack(stack_uuid = uuid, 
+                                private_key = private_key,
+                                action = action)
